from .qloader import qLoad
from .llibb import Crawler
import json
from .loadFlwd import selectFol, selectFol2, follow, login_
import random
import time

followed = []
logfile = open("followed.txt", "r")
followed = logfile.read().splitlines()
logfile.close()
logfile = open("followed.txt", "a")

stack = []

def getId(d):
    if "user" in d:
        return d["user"]["id"]
    elif "follow" in d:
        return d["follow"]["id"]
    else:
        return None

def addFollow(s):
    if s == None:
        return
    if len(stack) < 2500:
        if(s not in followed):
            followed.append(s)
            logfile.write(f"\n{s}")
            stack.append(s)

def selectList(t="entrystory", v={}):
    crawler = Crawler()

    a = crawler.postPage({
        "url": "https://playentry.org/graphql",
        "header": {
            "content-type": "application/json"
        },
        "body": json.dumps({
            "query": qLoad(f"select_{t}"),
            "variables":v
        })
    })

    j = json.loads(a)

    l = j["data"]["discussList"]["list"]

    return l

def selectES(n=3):
    return selectList("entrystory", {
        "category": "free",
        "searchType": "scroll",
        "term": "all",
        "discussType": "entrystory",
        "pageParam": {
            "display": n,
            "sort": "created"
        }
    })

def selectKT(n=3):
    return selectList("knowtips", {
        "category": "tips",
        "searchType": "scroll",
        "term": "all",
        "pageParam": {
            "display": n,
            "sort": "created"
        }
    })

def selectQA(n=3):
    return selectList("qna", {
        "category": "qna",
        "searchType": "page",
        "term": "all",
        "pageParam": {
            "start": 0,
            "display": n,
            "sort": "created"
        }
    })


def frame():
    if random.randint(0, 9) == 0:
        qrt = random.randint(1, 2)
    else:
        qrt = 0
    l = (selectES if qrt == 0 else selectKT if qrt == 1 else selectQA)()
    for i in l:
        addFollow(getId(i))
    
    qrt = random.randint(0, 1)
    if qrt == 0:
        l = selectFol()
    else:
        l = selectFol2()

    j = random.choice(l)
    qrt = random.randint(0, 1)
    if qrt == 0:
        l = selectFol(lid=getId(j))
    else:
        l = selectFol2(lid=getId(j))
    
    for i in l:
        addFollow(getId(i))

def followLast(n=5):
    if len(stack) > 0:
        t = min(n, len(stack))
        for i in range(t):
            j = stack.pop()
            follow(j)
            print(f"https://playentry.org/profile/{j} 님을 팔로우했어요!")

def run():
    while True:
        try:
            frame()
            if len(stack) > 1000:
                followLast(50)
            elif len(stack) > 100:
                followLast(15)
            else:
                followLast()
        except Exception as e:
            print(e)
        time.sleep(0.5)

def login(usr, pas):
    login_(pas, usr)
    




import json

qloaded = {}

def qLoad(s: str):
    if s in qloaded:
        return qloaded[s]
    else:
        f = open(f"{s}.txt", "r")
        t = json.loads(f.read())
        f.close()
        qloaded[s] = t
        return t



import requests, json

validURL = ["https://playentry.org/graphql", "https://playentry.org/rest/picture"]

def checkPostOptions(options):
    if "url" in options:
        url = options["url"].strip()
        if url not in validURL:
            return [False, "Url argument is not a valid graphql link"]
        else:
            if "header" not in options:
                return [False, "Missing header argument"]
            elif "body" not in options:
                return [False, "Missing body argument"]
            else:
                return [True, ""]
    else:
        return [False, "Missing url argument"]

class Crawler:
    def __init__(self) -> None:
        self.bot = requests.session()
        self.cookies = self.bot.cookies
        self.used = False

    def postPage(self, options, d401=False) -> str:
        if self.used == True:
            raise Exception("The current crawler is already used. Create a new crawler.")
        else:
            check = checkPostOptions(options)
            if check[0] == False:
                raise Exception(check[1])
            if d401:
                res = self.bot.post(options["url"], options["body"], headers=options["header"])
            else:
                res = self.bot.post(options["url"], options["body"], headers=options["header"])
            self.cookies = self.bot.cookies
            return res.text

    def postFile(self, options):
        if self.used == True:
            raise Exception("The current crawler is already used. Create a new crawler.")
        else:
            check = checkPostOptions(options)
            if check[0] == False:
                raise Exception(check[1])
            res = self.bot.post(options["url"], headers=options["header"], files=options["body"], data={})
            self.cookies = self.bot.cookies
            return res.status_code
    
    def saveCookies(self, filename: str) -> None:
        with open(filename, "w") as f:
            json.dump(requests.utils.dict_from_cookiejar(self.cookies), f)

    def loadCookies(self, filename: str) -> None:
        with open(filename, 'r') as f:
            cookies = requests.utils.cookiejar_from_dict(json.load(f))
            self.bot.cookies.update(cookies)

    def exportCookies(self) -> str:
        return json.dumps(requests.utils.dict_from_cookiejar(self.cookies))

    def loadCookiesFromStr(self, s: str):
        cookies = requests.utils.cookiejar_from_dict(json.loads(s))
        self.bot.cookies.update(cookies)



from .llibb import Crawler
from .qloader import qLoad
import json
from .login import loginAs

cookie = None

def selectFol(n=100, lid="621b0b7c294723001b1bc390"):
    if lid == None:
        return
    crawler = Crawler()

    a = crawler.postPage({
        "url": "https://playentry.org/graphql",
        "header": {
            "content-type": "application/json"
        },
        "body": json.dumps({
            "query": qLoad(f"select_followings"),
            "variables": {
                "user": lid,
                "pageParam":  {
                    "display": n
                }
            }
        })
    })

    j = json.loads(a)

    l = j["data"]["followings"]["list"]

    return l

def selectFol2(n=100, lid="621b0b7c294723001b1bc390"):
    if lid == None:
        return
    crawler = Crawler()

    a = crawler.postPage({
        "url": "https://playentry.org/graphql",
        "header": {
            "content-type": "application/json"
        },
        "body": json.dumps({
            "query": qLoad(f"select_followers"),
            "variables": {
                "user": lid,
                "pageParam":  {
                    "display": n
                }
            }
        })
    })

    j = json.loads(a)

    l = j["data"]["followers"]["list"]

    return l

def follow(lid: str):
    crawler = Crawler()
    crawler.loadCookiesFromStr(cookie)

    a = crawler.postPage({
        "url": "https://playentry.org/graphql",
        "header": {
            "content-type": "application/json"
        },
        "body": json.dumps({
            "query": qLoad(f"follow"),
            "variables": {
                "user": lid
            }
        })
    })

    j = json.loads(a)

    l = j["data"]

    return l

def login_(p, u):
    global cookie
    cookie = loginAs(p, u)


from .llibb import Crawler
import json
from .qloader import qLoad

def loginAs(pwd, usr):
    crawler = Crawler()

    a = crawler.postPage({
        "url": "https://playentry.org/graphql",
        "header": {
            "content-type": "application/json"
        },
        "body": json.dumps({
            "query": qLoad("login"),
            "variables":{
                "password": pwd,
                "username": usr,
                "rememberme": True
            }
        })
    })

    cookie: str = crawler.exportCookies()
    return cookie